      subroutine gridset

c  set up grid:  3-D spherical grid with variable spacing in r and theta
c  (set by exponents, rexp, texp
c  make a bunch of arrays necessary for find_wall subroutine

c history:
c 00/09/06 (baw): write subroutine
	
      implicit none
      include 'ds.txt'
      include 'out.txt'
      include 'grid.txt'
      include 'opacin.txt'

      real*8 dr,dt,dp,rad,phi,thet,pi,r2p,densd,dense
     $ ,cost,sint,pihalf,eps,tau,rming1,rming2,rming3,tiny
      real xyarr(200,200),x,y,xmax,dx,r
      integer ir,it,ip,nttmp,gflag,n,ix,iy

      pi=4.d0*datan(1.d0)
      r2p=2.d0*pi
      pihalf=pi/2.d0
      tiny=1.d-15

c     make grid include minimum and maximum values.

      print*, ' '
      print*, 'grid setup'

c     rgrid
      gflag=0
      rarr(1)=rmin
c     since there's empty space below rmine and rmind....
      if (rmind.lt.rmine) then
         gflag=1
         rarr(2)=rmind
         rming1=rmind
         rming2=rmine
      else if (rmind.gt.rmine) then
         gflag=2
         rarr(2)=rmine
         rming1=rmine
         rming2=rmind
      else if (rmind.eq.rmine) then
         gflag=3
         rarr(2)=rmine
         rming1=rmine
      endif
      dr=(rmax-rming1)/(dfloat(nrg-2))**rexp
c     print*,'rexp,dr,rmin',rexp,dr/au,rmin/au
      do ir=3,nrg
         rarr(ir)=rming1+dr*(dfloat(ir-2))**rexp
c     print*,'rarr ',ir,rarr(ir)/au
      enddo

c      do ir=1,nrg
c         print*,'rarr (in au) ',ir,rarr(ir)/au
c      enddo

      if (gflag.eq.1.or.gflag.eq.2) then
         call locate(rarr,nrg,rming2,ir)
         rarr(ir+1)=rming2
      endif

c     r-squared array
      do ir=1,nrg
         r2arr(ir)=rarr(ir)**2
c     print first few points of rarr
         if (ir.lt.10) then
            print*,'rarr/rmin,ir ',rarr(ir)/rmin,ir
         endif
      enddo
      print*,'rarr(nrg),r2arr(nrg)',rarr(nrg),r2arr(nrg)

      open(unit=15,file='rarr.dat',status='unknown')
      write(15,*) nrg,' = number of grid points in r'
      write(15,*) 'index     r/rstar  r(au)'
      do ir=1,nrg
         write(15,*) ir,(rarr(ir)),sngl(rarr(ir)/rmin)
      enddo
      close(15)

      if (ntg.gt.1) then
c     set up a tmp array going from 0 - 90 from equ. to pole
c     if rexp > 1, spacing will increase from equ. to pole
         nttmp=(ntg+1)/2
         pihalf=pi/2.d0
         dt=pihalf/(dfloat(nttmp-1))*texp
c     check: say ntg=19, nttmp=10, if texp=1, dt=10, correct
         do it=1,nttmp
            tmptharr(it)=dt*(dfloat(it-1))**texp
c     print*,'tmptharr ',tmptharr(it)
         enddo
         do it=1,nttmp
            thetarr(it)=pihalf-tmptharr(nttmp-it+1)
            if ((it+nttmp).le.ntg) thetarr(it+nttmp)=tmptharr(it)
     1           +pihalf+dt
         enddo
         eps=pihalf/100.
         do it=1,ntg
            costarr(it)=cos(thetarr(it))
            if(thetarr(it).gt.(pihalf-eps).and.thetarr(it).lt.
     1           (pihalf+eps)) 
     1           then
               tan2arr(it)=-1.d0
            else
               tan2arr(it)=tan(thetarr(it))**2
            endif
c     print*,'thetarr,costarr,tan2arr '
c     print*,thetarr(it),costarr(it),tan2arr(it)
         enddo
      else
         thetarr(1)=0.d0
      endif

      open(unit=15,file='tharr.dat',status='unknown')
      write(15,*) ntg,' = number of grid points in theta'
      write(15,*) 
     1     'index  thet(rad)  thet(deg)    cost     tan**2(thet)'
      do it=1,ntg
         write(15,*) it,sngl(thetarr(it)),sngl(thetarr(it)*180.d0/pi),
     1        sngl(costarr(it)),sngl(tan2arr(it))
      enddo
      close(15)

      if (npg.gt.1) then
         dp=r2p/dfloat(npg-1)
         do ip=1,npg
            phiarr(ip)=dp*(dfloat(ip-1))
            aarr(ip)=dsin(phiarr(ip))
            barr(ip)=-dcos(phiarr(ip))
c     print*,'phiarr, aarr, barr ',phiarr(ip),aarr(ip),barr(ip)
c     carr(ip)=
c     darr(ip)=
         enddo
      else
         dp=r2p
         ip=1
         phiarr(ip)=0.d0
         aarr(ip)=dsin(phiarr(ip))
         barr(ip)=-dcos(phiarr(ip))
      endif

      open(unit=15,file='phiarr.dat',status='unknown')
      write(15,*) npg
      do ip=1,npg
         write(15,*) phiarr(ip)*180.d0/pi
      enddo
      close(15)

c     calculate optical depth at thet=0, to rescale rho1,rho2,massenv
c     first set rho2 to an arbitrary value
      rho2=1.d-20
      rho1=rho1_2*rho2
      tau=0.d0
      thet=0.d0
      cost=1.d0-tiny
      sint=0.d0
      phi=0.d0
      do ir=1,nrg-1
         dr=rarr(ir+1)-rarr(ir)
         rad=0.5d0*(rarr(ir)+rarr(ir+1))
         call densenv(rad,rmin,thet,cost,pi,pihalf,f,phi,dense)
         tau=tau+kappa*(dense)*dr
      end do
c     now scale rho2 and rho1
      print*,'for rho2=1.d-20, tau= ',tau
      if (tau.gt.0.d0) then
         rho2=taumax0/tau*rho2
      else
         rho2=0.0
         print*,'no optical depth in envelope.  is that what you want?'
      endif
      rho1=rho1_2*rho2
      print*,'rho2, rho1, taumax0 ',rho2,rho1,taumax0

c     now calculate density in grid
      massenv=0.d0
      massdisk=0.d0
      do ir=1,nrg-1
         rad=0.5d0*(rarr(ir)+rarr(ir+1))
         dr=rarr(ir+1)-rarr(ir)
         if (ntg.gt.1) then     !2- or 3-D atmosphere
            do it=1,ntg-1
               thet=0.5d0*(thetarr(it)+thetarr(it+1))
               cost=cos(thet)
               sint=sin(thet)
               dt=thetarr(it+1)-thetarr(it)
               if (npg.gt.1) then
                  do ip=1,npg-1   !3-D atmosphere
                     phi=0.5d0*(phiarr(ip)+phiarr(ip+1))
                     dp=phiarr(ip+1)-phiarr(ip)
                     call densenv(rad,rmin,thet,cost,pi,pihalf,f,phi
     1                    ,dense)
                     massenv=massenv+dense*rad**2*sint*dt*dp*dr
                     call densdisk(rad,sint,cost,phi,densd)
                     massdisk=massdisk+densd*rad**2*sint*dt*dp*dr
                     densarr(ir,it,ip)=dense+densd
                  end do
               else           !2-D atmosphere
                  ip=1       
                  phi=0.d0
                  dp=r2p
                  call densenv(rad,rmin,thet,cost,pi,pihalf,f,phi
     1                 ,dense)
                  massenv=massenv+dense*rad**2*sint*dt*dp*dr
                  call densdisk(rad,sint,cost,phi,densd)
                  massdisk=massdisk+densd*rad**2*sint*dt*dp*dr
                  densarr(ir,it,ip)=dense+densd                  
               endif
            enddo
         else              !1-D atmosphere
            phi=0.d0
            ip=1
            it=1
            thet=0.d0
            cost=1.d0
            sint=0.d0
            call densenv(rad,rmin,thet,cost,pi,pihalf,f,phi,dense)
            massenv=massenv+dense*rad**2*dr*2.d0*r2p
            call densdisk(rad,sint,cost,phi,densd)
            massdisk=massdisk+densd*rad**2*dr*2.d0*r2p
            densarr(ir,it,ip)=dense+densd
c     print*,'densarr',densarr(ir,it,ip)
         endif
      end do
      massenv=massenv/1.989d33
      massdisk=massdisk/1.989d33
c     important note:  densarr(ir,it,ip) is the density halfway between
c     ir and ir+1, it and it+1, ip and ip+1; it is NOT the density at
c     ir,it,ip.

      print*, 'massenv (in solar units) ',massenv
      print*, 'massdisk from grid, compared to input ',massdisk,
     $     massd/1.989d33
      print*, ' '

c     write out rarr,thetarr,densarr at phi=0 so you can read them into
c     IDL.

c     integrate optical depth along the theta angles
c     at phi=0.
      open(unit=15,file='tau.dat',status='unknown')
      write(15,*) 'theta, optical depth'
      do it=1,ntg
         tau=0.d0
         thet=thetarr(it)
         cost=cos(thet)
         sint=sin(thet)
         phi=0.d0
         do ir=1,nrg-1
            dr=rarr(ir+1)-rarr(ir)
            rad=0.5d0*(rarr(ir)+rarr(ir+1))
            call densenv(rad,rmin,thet,cost,pi,pihalf,f,phi,dense)
            call densdisk(rad,sint,cost,phi,densd)
            tau=tau+kappa*(densd+dense)*dr
         end do
         write(15,*) thetarr(it)*180.d0/pi,tau
      end do
      close(15)


c     write out density array to read in IDL
c     make the cartesian grid here (fortran is faster than IDL)
c     this makes a grid in the x-z plane
      n=200
      xmax=sngl(rarr(nrg))
      dx=xmax/float(n)
      do ix=1,n
         x=dx*(float(ix)-0.5)
      do iy=1,n
         y=dx*(float(iy)-0.5)
         r=sqrt(x**2+y**2)
         thet=atan2(y,x)
         call locate(rarr,nrg,dble(r),ir)
         if (ntg.gt.1) then
            call locate(thetarr,ntg,(thet),it)
         else
            it=1
         endif
         xyarr(ix,iy)=sngl(densarr(ir,it,1))
      end do
      end do
      open(unit=15,file='densxz.dat',status='unknown')
      do ix=1,n
         write(15,*) (xyarr(ix,iy),iy=1,n)
      enddo
      close(15)

c     write out array in x-y plane at z=.5*rmax to see phi dependence
      thet=pi/4.d0
      if (it.gt.1) then
         call locate(thetarr,ntg,(thet),it)
      else
         it=1
      endif
      do ix=1,n
         x=dx*(float(ix)-0.5)
      do iy=1,n
         y=dx*(float(iy)-0.5)
         r=sqrt(x**2+y**2)
         phi=atan2(y,x)
         call locate(rarr,nrg,dble(r),ir)
         if (npg.gt.1) then
            call locate(phiarr,npg,(phi),ip)
         else
            ip=1
         endif
         xyarr(ix,iy)=sngl(densarr(ir,it,ip))
      end do
      end do
      open(unit=15,file='densxy.dat',status='unknown')
      do ix=1,n
         write(15,*) (xyarr(ix,iy),iy=1,n)
      enddo
      close(15)

      end


      







